import tensorflow_datasets as tfds
import numpy as np
import tensorflow as tf
import os
import collections

_Y10_IMAGES_TRAIN_URL = "images_Y10_train.npy?download=1"
_Y1_IMAGES_TRAIN_URL = "images_Y1_train.npy?download=1"
_LABELS_TRAIN_URL = "labels_train.npy?download=1"

_Y10_IMAGES_VALID_URL = "images_Y10_valid.npy?download=1"
_Y1_IMAGES_VALID_URL = "images_Y1_valid.npy?download=1"
_LABELS_VALID_URL = "labels_valid.npy?download=1"

_Y10_IMAGES_TEST_URL = "images_Y10_test.npy?download=1"
_Y1_IMAGES_TEST_URL = "images_Y1_test.npy?download=1"
_LABELS_TEST_URL = "labels_test.npy?download=1"

_MLSST_IMAGE_SIZE = 100
_MLSST_IMAGE_SHAPE = (_MLSST_IMAGE_SIZE, _MLSST_IMAGE_SIZE, 3)

_URL = "https://zenodo.org/record/5514180/files/"

_DATA_OPTIONS = ['Y1', 'Y10']


class MLSSTConfig(tfds.core.BuilderConfig):
    """BuilderConfig for MLSST"""

    def __init__(self, *, data=None, **kwargs):
        """Constructs a MLSSTConfig.
        Args:
          data: `str`, one of `_DATA_OPTIONS`.
          **kwargs: keyword arguments forwarded to super.
        """
        if data not in _DATA_OPTIONS:
            raise ValueError("data must be one of %s" % _DATA_OPTIONS)

        super(MLSSTConfig, self).__init__(**kwargs)
        self.data = data


class MLSST(tfds.core.GeneratorBasedBuilder):
    """Mock LSST dataset"""

    BUILDER_CONFIGS = [
        MLSSTConfig(
            name=config_name,
            version=tfds.core.Version("1.0.1"),
            data=config_name,
        ) for config_name in _DATA_OPTIONS
    ]

    num_classes = 3

    def _info(self):
        return tfds.core.DatasetInfo(
            builder=self,
            description=("The Mock LSST dataset consists of 2 sets of survey-emulating images, by applying an  "
                         "exposure time directly to raw images: low-noise 10 year (Y10) survey, high-noise 1 year (Y1)."
                         "The data is split into 3 sets: training, validation and testing."),
            features=tfds.features.FeaturesDict({
                "image": tfds.features.Tensor(shape=_MLSST_IMAGE_SHAPE, dtype=tf.float32),
                "label": tfds.features.ClassLabel(num_classes=self.num_classes),
            }),
            supervised_keys=("image", "label"),
            homepage="https://zenodo.org/record/5514180#.Yt-gRnbMIjJ",
        )

    @property
    def _mlsst_info(self):
        return MLSSTInfo(
            name=self.name,
            url=_URL,
            Y10_train_files=_Y10_IMAGES_TRAIN_URL,
            Y1_train_files=_Y1_IMAGES_TRAIN_URL,
            Y10_validation_files=_Y10_IMAGES_VALID_URL,
            Y1_validation_files=_Y1_IMAGES_VALID_URL,
            Y10_test_files=_Y10_IMAGES_TEST_URL,
            Y1_test_files=_Y1_IMAGES_TEST_URL,
            train_label_files=_LABELS_TRAIN_URL,
            validation_label_files=_LABELS_VALID_URL,
            test_label_files=_LABELS_TEST_URL,
            label_keys=["label"],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        if self.builder_config.name == "Y10":

            train_image_url = self._mlsst_info.Y10_train_files
            valid_image_url = self._mlsst_info.Y10_validation_files
            test_image_url = self._mlsst_info.Y10_test_files

        elif self.builder_config.name == "Y1":

            train_image_url = self._mlsst_info.Y1_train_files
            valid_image_url = self._mlsst_info.Y1_validation_files
            test_image_url = self._mlsst_info.Y1_test_files

        train_label_url = self._mlsst_info.train_label_files
        valid_label_url = self._mlsst_info.validation_label_files
        test_label_url = self._mlsst_info.test_label_files

        return {
            'train': self._generate_examples(
                image_path=dl_manager.download(os.path.join(self._mlsst_info.url, train_image_url)),
                label_path=dl_manager.download(os.path.join(self._mlsst_info.url, train_label_url)),
            ),
            'validation': self._generate_examples(
                image_path=dl_manager.download(os.path.join(self._mlsst_info.url, valid_image_url)),
                label_path=dl_manager.download(os.path.join(self._mlsst_info.url, valid_label_url)),
            ),
            'test': self._generate_examples(
                image_path=dl_manager.download(os.path.join(self._mlsst_info.url, test_image_url)),
                label_path=dl_manager.download(os.path.join(self._mlsst_info.url, test_label_url)),
            ),
        }

    def _generate_examples(self, image_path, label_path):
        with tf.io.gfile.GFile(image_path, "rb") as f:
            images = np.transpose(np.load(f), (0, 2, 3, 1)).astype(np.float32)
        with tf.io.gfile.GFile(label_path, "rb") as f:
            labels = np.argmax(np.load(f), axis=1)
        for i, (image, label) in enumerate(zip(images, labels)):
            record = {
                "image": image,
                "label": label,
            }
            yield i, record


class MLSSTInfo(
    collections.namedtuple("_MLSSTInfo", [
        "name",
        "url",
        "Y10_train_files",
        "Y1_train_files",
        "Y10_validation_files",
        "Y1_validation_files",
        "Y10_test_files",
        "Y1_test_files",
        "train_label_files",
        "validation_label_files",
        "test_label_files",
        "label_keys"
    ])):
    """Contains the information necessary to generate a MLSST dataset.
    Attributes:
      name (str): name of dataset.
      url (str): data URL.
      label_keys (list<str>): names of the label keys in the data.
    """



"""Module containing the strong lensing challenge dataset."""

import tensorflow as tf
import tensorflow_datasets as tfds
import os
import collections
import h5py
import numpy as np

_SLC_IMAGE_SIZE = 101

_DATA_OPTIONS = ['space']


class SLCConfig(tfds.core.BuilderConfig):
    """BuilderConfig for SLC"""

    def __init__(self, *, data=None, num_channels=1, **kwargs):
        """Constructs a SLCConfig.
        Args:
          data: `str`, one of `_DATA_OPTIONS`.
          **kwargs: keyword arguments forwarded to super.
        """
        if data not in _DATA_OPTIONS:
            raise ValueError("data must be one of %s" % _DATA_OPTIONS)

        super(SLCConfig, self).__init__(**kwargs)
        self.data = data
        self.image_shape = (_SLC_IMAGE_SIZE, _SLC_IMAGE_SIZE, num_channels)


class SLC(tfds.core.GeneratorBasedBuilder):
    """SLC"""

    VERSION = tfds.core.Version("1.0.0")

    BUILDER_CONFIGS = [
        SLCConfig(
            name='space',
            version=tfds.core.Version("1.0.0"),
            data='space',
            num_channels=1,
        ),
    ]

    def _info(self):
        return tfds.core.DatasetInfo(
            builder=self,
            description=("The strong lensing challenge dataset consists of 101x101 "
                         "images in 2 classes. There "
                         "are 20000 training images and 20000 test images."),
            features=tfds.features.FeaturesDict({
                "image": tfds.features.Tensor(shape=self.builder_config.image_shape, dtype=tf.float32),
                "label": tfds.features.ClassLabel(num_classes=2),
            }),
            supervised_keys=("image", "label"),
            homepage="http://metcalf1.difa.unibo.it/blf-portal/gg_challenge.html",
        )

    @property
    def _slc_info(self):
        return SLCInfo(
            name=self.name,
            url="https://storage.googleapis.com/strong-lensing-challenge",
            space_file="strong-lensing-space-based-challenge1.tar.gz",
            ground_file="",
            train_files=[
                "strong-lensing-space-based-challenge1/train1.h5",
            ],
            test_files=[
                "strong-lensing-space-based-challenge1/test1.h5"
            ],
            image_key="data",
            label_key="is_lens",
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        if self.builder_config.name == "space":

            slc_path = dl_manager.download_and_extract(os.path.join(self._slc_info.url, self._slc_info.space_file))

        slc_info = self._slc_info

        # Define the splits
        def gen_filenames(filenames):
            for f in filenames:
                yield os.path.join(slc_path, f)

        return {
            'train': self._generate_examples(filepaths=gen_filenames(slc_info.train_files)),
            'test': self._generate_examples(filepaths=gen_filenames(slc_info.test_files)),
        }

    def _generate_examples(self, filepaths):
        for path in filepaths:
            with h5py.File(path, "r") as f:
                images = f[self._slc_info.image_key][:]
                labels = f[self._slc_info.label_key][:]
            for i, (image, label) in enumerate(zip(images, labels)):
                if len(image.shape) == 2:
                    image = np.expand_dims(image, -1)
                record = {
                    "image": image,
                    "label": label,
                }
                yield i, record


class SLCInfo(
    collections.namedtuple("_SLCInfo", [
        "name",
        "url",
        "space_file",
        "ground_file",
        "train_files",
        "test_files",
        "image_key",
        "label_key",
    ])):
    """Contains the information necessary to generate a SLC dataset.
       Attributes:
         name (str): name of dataset.
         url (str): data URL.
         train_files (list<str>): name of training files within `prefix`.
         label_files (list<str>): names of the label files in the data.
         label_keys (list<str>): names of the label keys in the data.
       """



"""Module containing the Mirabest dataset."""

import tensorflow_datasets as tfds
import os
import collections
import pickle
import numpy as np

_MIRABEST_IMAGE_SIZE = 150
_MIRABEST_IMAGE_SHAPE = (_MIRABEST_IMAGE_SIZE, _MIRABEST_IMAGE_SIZE, 1)

_DATA_OPTIONS = ['all', 'confident', 'uncertain']


class MirabestConfig(tfds.core.BuilderConfig):
    """BuilderConfig for Mirabest"""

    def __init__(self, *, data=None, num_classes=None, class_keys=None, label_names=None, **kwargs):
        """Constructs a MirabestConfig.
        Args:
          data: `str`, one of `_DATA_OPTIONS`.
          **kwargs: keyword arguments forwarded to super.
        """
        if data not in _DATA_OPTIONS:
            raise ValueError("data must be one of %s" % _DATA_OPTIONS)

        super(MirabestConfig, self).__init__(**kwargs)
        self.data = data
        self.num_classes = num_classes
        self.class_keys = class_keys
        self.label_names = label_names


class MIRABEST(tfds.core.GeneratorBasedBuilder):
    """MiraBest"""

    VERSION = tfds.core.Version("1.0.1")

    BUILDER_CONFIGS = [
        MirabestConfig(
            name='all',
            version=tfds.core.Version("1.0.1"),
            data='all',
            num_classes=10,
            class_keys={
                0: 0,
                1: 1,
                2: 2,
                3: 3,
                4: 4,
                5: 5,
                6: 6,
                7: 7,
                8: 8,
                9: 9,
            },
            label_names=None
        ),
        MirabestConfig(
            name='confident',
            version=tfds.core.Version("1.0.1"),
            data='confident',
            num_classes=2,
            class_keys={
                0: 0,
                1: 0,
                2: 0,
                5: 1,
                6: 1
            },
            label_names=['FR1', 'FR2']
        ),
        MirabestConfig(
            name='uncertain',
            version=tfds.core.Version("1.0.1"),
            data='uncertain',
            num_classes=2,
            class_keys={
                3: 0,
                4: 0,
                7: 1,
            },
            label_names=['FR1', 'FR2']
        ),
    ]

    def _info(self):
        return tfds.core.DatasetInfo(
            builder=self,
            description=("The MiraBest dataset consists of 1256 150x150 "
                         "images in 10 classes. There "
                         "are 1099 training images and 157 test images."),
            features=tfds.features.FeaturesDict({
                "id": tfds.features.Text(),
                "image": tfds.features.Image(shape=_MIRABEST_IMAGE_SHAPE),
                "label": tfds.features.ClassLabel(num_classes=self.builder_config.num_classes),
            }),
            supervised_keys=("image", "label"),
            homepage="https://zenodo.org/record/5588282#.Yr7X8HbMIjJ",
        )

    @property
    def _mirabest_info(self):
        return MiraBestInfo(
            name=self.name,
            url="http://www.jb.man.ac.uk/research/MiraBest/full_dataset/MiraBest_full_batches.tar.gz",
            train_files=[
                "data_batch_1", "data_batch_2", "data_batch_3",
                "data_batch_4", "data_batch_5", "data_batch_6",
                "data_batch_7"
            ],
            test_files=["test_batch"],
            prefix="batches",
            label_files=["batches.meta"],
            label_keys=["label"],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        mirabest_path = dl_manager.download_and_extract(self._mirabest_info.url)
        mirabest_info = self._mirabest_info

        mirabest_path = os.path.join(mirabest_path, mirabest_info.prefix)

        # Load the label names
        for label_key, label_file in zip(mirabest_info.label_keys,
                                         mirabest_info.label_files):
            labels_path = os.path.join(mirabest_path, label_file)
            with open(labels_path, 'rb') as f:
                data = pickle.load(f, encoding='latin1')
            if self.builder_config.label_names is not None:
                self.info.features[label_key].names = self.builder_config.label_names
            else:
                self.info.features[label_key].names = data['label_names']

        # Define the splits
        def gen_filenames(filenames):
            for f in filenames:
                yield os.path.join(mirabest_path, f)

        return {
            tfds.Split.TRAIN:
                self._generate_examples("train_",
                                        gen_filenames(mirabest_info.train_files)),
            tfds.Split.TEST:
                self._generate_examples("test_",
                                        gen_filenames(mirabest_info.test_files)),
        }

    def _generate_examples(self, split_prefix, filepaths):
        label_keys = self._mirabest_info.label_keys
        index = 0  # Using index as key since data is always loaded in same order.
        for path in filepaths:
            for label, np_image in _load_data(path):
                if label not in self.builder_config.class_keys:
                    continue
                record = dict(zip(label_keys, [self.builder_config.class_keys[label]]))
                # Note: "id" is only provided for the user convenience. To shuffle the
                # dataset we use `index`, so that the sharding is compatible with
                # earlier versions.
                record["id"] = "{}{:05d}".format(split_prefix, index)
                record["image"] = np_image
                yield index, record
                index += 1


class MiraBestInfo(
    collections.namedtuple("_MiraBestInfo", [
        "name",
        "url",
        "prefix",
        "train_files",
        "test_files",
        "label_files",
        "label_keys",
    ])):
    """Contains the information necessary to generate a Mirabest dataset.
    Attributes:
      name (str): name of dataset.
      url (str): data URL.
      prefix (str): path prefix within the downloaded and extracted file to look
        for `train_files` and `test_files`.
      train_files (list<str>): name of training files within `prefix`.
      test_files (list<str>): name of test files within `prefix`.
      label_files (list<str>): names of the label files in the data.
      label_keys (list<str>): names of the label keys in the data.
    """


def _load_data(path):
    """Yields (label, np_image) tuples."""
    with open(path, 'rb') as f:
        data = pickle.load(f)
    for i in range(len(data['labels'])):
        yield data['labels'][i], np.expand_dims(data['data'][i], -1)



"""Utility functions and classes used by astro datasets."""
import tensorflow as tf
import tensorflow_datasets as tfds
from tensorflow_datasets.core.features import FeaturesDict, Tensor


class AstroTsDatasetInfo(tfds.core.DatasetInfo):
    """Dataset info for astro time series datasets."""

    time_dtype = tf.float32
    static_dtype = tf.float32
    timeseries_dtype = tf.float32
    object_id_dtype = tf.uint32
    redshift_dtype = tf.float32

    def __init__(self, builder, targets, default_target,
                 static_names=None, timeseries_names=None,
                 description=None, homepage=None, citation=None):
        """Dataset info for astro time series datasets.

        Ensures all datasets follow a similar structure and can be used
        (almost) interchangably.

        Args:
            builder: Builder class associated with this dataset info.
            targets: Dictionary of endpoints.
            static_names: Names of the static measurements.
            timeseries_names: Names of the time series measurements.
            description: Dataset description.
            homepage: Homepage of dataset.
            citation: Citation of dataset.

        """
        self.has_static = static_names is not None
        self.has_timeseries = timeseries_names is not None
        self.default_target = default_target

        metadata = tfds.core.MetadataDict()
        features_dict = {
            'time': Tensor(shape=(None,), dtype=self.time_dtype)
        }
        static_is_categorical = []
        timeseries_is_categorical = []
        if self.has_static:
            metadata['static_names'] = static_names
            static_is_categorical.extend(
                ['=' in static_name for static_name in static_names])
            features_dict['static'] = Tensor(
                shape=(len(static_names),),
                dtype=self.static_dtype)
            features_dict['static_errors'] = Tensor(
                shape=(len(static_names),),
                dtype=self.static_dtype)
        if self.has_timeseries:
            metadata['value_names'] = timeseries_names
            timeseries_is_categorical.extend(
                ['=' in name for name in timeseries_names])
            features_dict['values'] = Tensor(
                shape=(None, len(timeseries_names),),
                dtype=self.timeseries_dtype)
            features_dict['value_errors'] = Tensor(
                shape=(None, len(timeseries_names),),
                dtype=self.timeseries_dtype)

        metadata['static_categorical_indicator'] = static_is_categorical
        metadata['values_categorical_indicator'] = timeseries_is_categorical
        metadata['has_errors'] = True

        features_dict['targets'] = targets
        features_dict['metadata'] = {
            'object_id': self.object_id_dtype,
            'redshift': self.redshift_dtype
        }
        features_dict = FeaturesDict(features_dict)
        # TODO: If we are supposed to return raw values, we cannot make
        # a supervised dataset
        if builder.output_raw:
            supervised_keys = None
        else:
            supervised_keys = ('combined', 'target')

        super().__init__(
            builder=builder,
            description=description, homepage=homepage, citation=citation,
            features=features_dict,
            supervised_keys=supervised_keys,
            metadata=metadata
        )


class AstroTsDatasetBuilder(tfds.core.GeneratorBasedBuilder):
    """Builder class for astro time series datasets."""

    def __init__(self, output_raw=False, **kwargs):
        self.output_raw = output_raw
        # Hack to allow default target to be changed when loading dataset
        if 'default_target' in kwargs:
            self.default_target = kwargs['default_target']
            kwargs.pop('default_target')
        super().__init__(**kwargs)

    def _as_dataset(self, **kwargs):
        """Evtl. transform categorical covariates into one-hot encoding."""
        dataset = super()._as_dataset(**kwargs)
        if self.output_raw:
            return dataset

        has_static = self.info.has_static
        has_timeseries = self.info.has_timeseries

        def preprocess_output(instance):
            if has_static:
                static = instance['static']
                static_errors = instance['static_errors']
            else:
                static = None
                static_errors = None
            if has_timeseries:
                values = instance['values']
                value_errors = instance['value_errors']
            else:
                values = None

            time = instance['time']

            measurements = tf.math.is_finite(values)
            length = tf.shape(time)[0]
            return {
                'combined': (
                    static,
                    time,
                    values,
                    measurements,
                    length,
                    static_errors,
                    value_errors,
                ),
                'target': instance['targets'][self.default_target],
                'metadata': instance['metadata']
            }

        return dataset.map(
            preprocess_output,
            num_parallel_calls=tf.data.experimental.AUTOTUNE
        )




import tensorflow_datasets as tfds
from tensorflow_datasets.core.constants import DATA_DIR
import os
import collections
import numpy as np
import tensorflow as tf

# CMD constants
_CMD_URL = "https://users.flatironinstitute.org/~fvillaescusa/priv/DEPnzxoWlaTQ6CjrXqsm0vYi8L7Jy/CMD/2D_maps/data/"
# SIMBA and IllustrisTNG parameters
_CMD_SIMBA_PARAMS_FILENAME = "params_SIMBA.txt"
_CMD_ILLUS_PARAMS_FILENAME = "params_IllustrisTNG.txt"
# N-body corresponding parameters
_CMD_N_SIMBA_PARAMS_FILENAME = "params_Nbody_SIMBA.txt"
_CMD_N_ILLUS_PARAMS_FILENAME = "params_Nbody_IllustrisTNG.txt"
# N-body total matter density data
_CMD_N_ILLUS_CV_MTOT_DATA_FILENAME = "Maps_Mtot_Nbody_IllustrisTNG_CV_z=0.00.npy"
_CMD_N_ILLUS_LH_MTOT_DATA_FILENAME = "Maps_Mtot_Nbody_IllustrisTNG_LH_z=0.00.npy"
_CMD_N_SIMBA_CV_MTOT_DATA_FILENAME = "Maps_Mtot_Nbody_SIMBA_CV_z=0.00.npy"
_CMD_N_SIMBA_LH_MTOT_DATA_FILENAME = "Maps_Mtot_Nbody_SIMBA_LH_z=0.00.npy"
# IllustrisTNG magenetic field data
_CMD_ILLUS_CV_B_DATA_FILENAME = "Maps_B_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_B_DATA_FILENAME = "Maps_B_IllustrisTNG_LH_z=0.00.npy"
# Neutral hydrogen density field data
_CMD_ILLUS_CV_HI_DATA_FILENAME = "Maps_HI_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_HI_DATA_FILENAME = "Maps_HI_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_HI_DATA_FILENAME = "Maps_HI_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_HI_DATA_FILENAME = "Maps_HI_SIMBA_LH_z=0.00.npy"
# Dark matter density field data
_CMD_ILLUS_CV_MCDM_DATA_FILENAME = "Maps_Mcdm_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_MCDM_DATA_FILENAME = "Maps_Mcdm_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_MCDM_DATA_FILENAME = "Maps_Mcdm_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_MCDM_DATA_FILENAME = "Maps_Mcdm_SIMBA_LH_z=0.00.npy"
# Gas density field data
_CMD_ILLUS_CV_MGAS_DATA_FILENAME = "Maps_Mgas_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_MGAS_DATA_FILENAME = "Maps_Mgas_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_MGAS_DATA_FILENAME = "Maps_Mgas_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_MGAS_DATA_FILENAME = "Maps_Mgas_SIMBA_LH_z=0.00.npy"
# Magnesium over iron field data
_CMD_ILLUS_CV_MGFE_DATA_FILENAME = "Maps_MgFe_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_MGFE_DATA_FILENAME = "Maps_MgFe_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_MGFE_DATA_FILENAME = "Maps_MgFe_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_MGFE_DATA_FILENAME = "Maps_MgFe_SIMBA_LH_z=0.00.npy"
# Stellar mass density field data
_CMD_ILLUS_CV_MSTAR_DATA_FILENAME = "Maps_Mstar_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_MSTAR_DATA_FILENAME = "Maps_Mstar_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_MSTAR_DATA_FILENAME = "Maps_Mstar_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_MSTAR_DATA_FILENAME = "Maps_Mstar_SIMBA_LH_z=0.00.npy"
# Total mass density field data
_CMD_ILLUS_CV_MTOT_DATA_FILENAME = "Maps_Mtot_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_MTOT_DATA_FILENAME = "Maps_Mtot_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_MTOT_DATA_FILENAME = "Maps_Mtot_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_MTOT_DATA_FILENAME = "Maps_Mtot_SIMBA_LH_z=0.00.npy"
# Electron number density field data
_CMD_ILLUS_CV_NE_DATA_FILENAME = "Maps_ne_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_NE_DATA_FILENAME = "Maps_ne_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_NE_DATA_FILENAME = "Maps_ne_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_NE_DATA_FILENAME = "Maps_ne_SIMBA_LH_z=0.00.npy"
# Gas pressure field data
_CMD_ILLUS_CV_P_DATA_FILENAME = "Maps_P_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_P_DATA_FILENAME = "Maps_P_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_P_DATA_FILENAME = "Maps_P_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_P_DATA_FILENAME = "Maps_P_SIMBA_LH_z=0.00.npy"
# Gas temperature field data
_CMD_ILLUS_CV_T_DATA_FILENAME = "Maps_T_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_T_DATA_FILENAME = "Maps_T_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_T_DATA_FILENAME = "Maps_T_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_T_DATA_FILENAME = "Maps_T_SIMBA_LH_z=0.00.npy"
# Dark matter velocity field data
_CMD_ILLUS_CV_VCDM_DATA_FILENAME = "Maps_Vcdm_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_VCDM_DATA_FILENAME = "Maps_Vcdm_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_VCDM_DATA_FILENAME = "Maps_Vcdm_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_VCDM_DATA_FILENAME = "Maps_Vcdm_SIMBA_LH_z=0.00.npy"
# Gas velocity field data
_CMD_ILLUS_CV_VGAS_DATA_FILENAME = "Maps_Vgas_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_VGAS_DATA_FILENAME = "Maps_Vgas_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_VGAS_DATA_FILENAME = "Maps_Vgas_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_VGAS_DATA_FILENAME = "Maps_Vgas_SIMBA_LH_z=0.00.npy"
# Gas metallicity field data
_CMD_ILLUS_CV_Z_DATA_FILENAME = "Maps_Z_IllustrisTNG_CV_z=0.00.npy"
_CMD_ILLUS_LH_Z_DATA_FILENAME = "Maps_Z_IllustrisTNG_LH_z=0.00.npy"
_CMD_SIMBA_CV_Z_DATA_FILENAME = "Maps_Z_SIMBA_CV_z=0.00.npy"
_CMD_SIMBA_LH_Z_DATA_FILENAME = "Maps_Z_SIMBA_LH_z=0.00.npy"
# CMD image sizes
_CMD_IMAGE_SIZE = 256
_CMD_IMAGE_SHAPE = (_CMD_IMAGE_SIZE, _CMD_IMAGE_SIZE, 1)

_CMD_PARAMETERS = ['omegam', 'sigma8', 'asn1', 'aagn1', 'asn2', 'aagn2']
_CMD_FIELDS = ['Mtot', 'Mtot_Nbody', 'B', 'HI', 'Mcdm', 'Mgas', 'MgFe', 'Mstar', 'ne', 'P', 'T', 'Vcdm',
               'Vgas', 'Z']
_CMD_SIMULATIONS = ['IllustrisTNG', 'SIMBA']
_CMD_SIM_SET = ['CV', 'LH']


class CMD(tfds.core.GeneratorBasedBuilder):
    """CAMELS Multifield Dataset"""

    URL = _CMD_URL
    VERSION = tfds.core.Version("1.0.4")

    def __init__(self, simulation, field, parameters=None, *kwargs, **kwds):
        assert simulation in _CMD_SIMULATIONS, 'Not a valid simulation'
        assert field in _CMD_FIELDS, 'Not a valid field'
        self.simulation = simulation
        self.field = field
        if parameters is not None:
            assert len(parameters) > 0, 'No parameters given'
            for p in parameters:
                assert p in _CMD_PARAMETERS, 'Not a valid parameter'
            self.parameters = parameters
        else:
            self.parameters = _CMD_PARAMETERS
        self.parameter_indices = [_CMD_PARAMETERS.index(p) for p in self.parameters]
        kwds['data_dir'] = os.path.join(DATA_DIR, '_'.join([simulation, field] + self.parameters))
        super().__init__(*kwargs, **kwds)

    def _info(self):
        return tfds.core.DatasetInfo(
            builder=self,
            description=("The CAMELS Multifield dataset consists of 3 groups "
                         "indicating the type of simulation used to create the data."
                         "IllustrisTNG are magneto-hydrodynamic simulations."
                         "SIMBA are hydrodynamic simulations."
                         "There are corresponding N-body simulations for each"
                         "(magneto-)hydrodynamic simulation."),
            features=tfds.features.FeaturesDict({
                "image": tfds.features.Tensor(shape=_CMD_IMAGE_SHAPE, dtype=tf.float32),
                'label': tfds.features.Tensor(shape=(len(self.parameters),), dtype=tf.float32),
            }),
            supervised_keys=("image", "label"),
            homepage="https://camels-multifield-dataset.readthedocs.io/en/latest/index.html",
        )

    @property
    def _cmd_info(self):
        return CMDInfo(
            name=self.name,
            url="https://users.flatironinstitute.org/~fvillaescusa/priv/DEPnzxoWlaTQ6CjrXqsm0vYi8L7Jy/CMD/2D_maps/data/",
            train_files=[_CMD_N_ILLUS_CV_MTOT_DATA_FILENAME, _CMD_N_ILLUS_LH_MTOT_DATA_FILENAME,
                         _CMD_N_SIMBA_CV_MTOT_DATA_FILENAME, _CMD_N_SIMBA_LH_MTOT_DATA_FILENAME,
                         _CMD_ILLUS_CV_B_DATA_FILENAME, _CMD_ILLUS_LH_B_DATA_FILENAME, _CMD_ILLUS_CV_HI_DATA_FILENAME,
                         _CMD_ILLUS_LH_HI_DATA_FILENAME, _CMD_SIMBA_CV_HI_DATA_FILENAME, _CMD_SIMBA_LH_HI_DATA_FILENAME,
                         _CMD_ILLUS_CV_MCDM_DATA_FILENAME, _CMD_ILLUS_LH_MCDM_DATA_FILENAME,
                         _CMD_SIMBA_CV_MCDM_DATA_FILENAME, _CMD_SIMBA_LH_MCDM_DATA_FILENAME,
                         _CMD_ILLUS_CV_MGAS_DATA_FILENAME, _CMD_ILLUS_LH_MGAS_DATA_FILENAME,
                         _CMD_SIMBA_CV_MGAS_DATA_FILENAME, _CMD_SIMBA_LH_MGAS_DATA_FILENAME,
                         _CMD_ILLUS_CV_MGFE_DATA_FILENAME, _CMD_ILLUS_LH_MGFE_DATA_FILENAME,
                         _CMD_SIMBA_CV_MGFE_DATA_FILENAME, _CMD_SIMBA_LH_MGFE_DATA_FILENAME,
                         _CMD_ILLUS_CV_MSTAR_DATA_FILENAME, _CMD_ILLUS_LH_MSTAR_DATA_FILENAME,
                         _CMD_SIMBA_CV_MSTAR_DATA_FILENAME, _CMD_SIMBA_LH_MSTAR_DATA_FILENAME,
                         _CMD_ILLUS_CV_MTOT_DATA_FILENAME, _CMD_ILLUS_LH_MTOT_DATA_FILENAME,
                         _CMD_SIMBA_CV_MTOT_DATA_FILENAME, _CMD_SIMBA_LH_MTOT_DATA_FILENAME,
                         _CMD_ILLUS_CV_NE_DATA_FILENAME, _CMD_ILLUS_LH_NE_DATA_FILENAME, _CMD_SIMBA_CV_NE_DATA_FILENAME,
                         _CMD_SIMBA_LH_NE_DATA_FILENAME,
                         _CMD_ILLUS_CV_P_DATA_FILENAME, _CMD_ILLUS_LH_P_DATA_FILENAME, _CMD_SIMBA_CV_P_DATA_FILENAME,
                         _CMD_SIMBA_LH_P_DATA_FILENAME,
                         _CMD_ILLUS_CV_T_DATA_FILENAME, _CMD_ILLUS_LH_T_DATA_FILENAME, _CMD_SIMBA_CV_T_DATA_FILENAME,
                         _CMD_SIMBA_LH_T_DATA_FILENAME,
                         _CMD_ILLUS_CV_VCDM_DATA_FILENAME, _CMD_ILLUS_LH_VCDM_DATA_FILENAME,
                         _CMD_SIMBA_CV_VCDM_DATA_FILENAME, _CMD_SIMBA_LH_VGAS_DATA_FILENAME,
                         _CMD_ILLUS_CV_VGAS_DATA_FILENAME, _CMD_ILLUS_LH_VGAS_DATA_FILENAME,
                         _CMD_SIMBA_CV_VGAS_DATA_FILENAME, _CMD_SIMBA_LH_VGAS_DATA_FILENAME,
                         _CMD_ILLUS_CV_Z_DATA_FILENAME, _CMD_ILLUS_LH_Z_DATA_FILENAME, _CMD_SIMBA_CV_Z_DATA_FILENAME,
                         _CMD_SIMBA_LH_Z_DATA_FILENAME],
            label_files=[_CMD_SIMBA_PARAMS_FILENAME, _CMD_ILLUS_PARAMS_FILENAME,
                         _CMD_N_SIMBA_PARAMS_FILENAME, _CMD_N_ILLUS_PARAMS_FILENAME],
            label_keys=["label"],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        params_file = 'params_' + self.simulation + '.txt'
        assert params_file in self._cmd_info.label_files, 'Invalid parameter file'
        cmd_label_path = dl_manager.download(os.path.join(_CMD_URL, params_file))
        data_file = 'Maps_' + self.field + '_' + self.simulation + '_LH_z=0.00.npy'
        assert data_file in self._cmd_info.train_files, 'Invalid data file'
        cmd_data_path = dl_manager.download(os.path.join(_CMD_URL, data_file))
        return {
            'train': self._generate_examples(
                images_path=cmd_data_path,
                label_path=cmd_label_path,
            ),
        }

    def _generate_examples(self, images_path, label_path):
        with tf.io.gfile.GFile(images_path, "rb") as f:
            images = np.load(f)
            images = np.expand_dims(images, axis=-1).astype(np.float32)
        with tf.io.gfile.GFile(label_path, "rb") as f:
            labels = np.loadtxt(f)
            labels = np.repeat(labels, 15, axis=0)
            labels = labels[:, self.parameter_indices].astype(np.float32)
        for i, (image, label) in enumerate(zip(images, labels)):
            record = {
                "image": image,
                "label": label,
            }
            yield i, record


class CMDInfo(
    collections.namedtuple("_CMDInfo", [
        "name",
        "url",
        "train_files",
        "label_files",
        "label_keys",
    ])):
    """Contains the information necessary to generate a CMD dataset.
    Attributes:
      name (str): name of dataset.
      url (str): data URL.
      train_files (list<str>): name of training files within `prefix`.
      label_files (list<str>): names of the label files in the data.
      label_keys (list<str>): names of the label keys in the data.
    """



# Tensorflow datasets of astronomical time series
import astro_datasets.time_series.spcc
import astro_datasets.time_series.plasticc

# Tensorflow datasets of astronomical images
import astro_datasets.images.mirabest
import astro_datasets.images.cmd
import astro_datasets.images.mlsst
import astro_datasets.images.slc

builders = [
    'spcc',
    'plasticc',
    'mirabest',
    'cmd',
    'mlsst',
    'slc',
]

__version__ = '0.0.18'



"""Module containing the Supernova Photometric Classification Challenge 2010."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
from collections.abc import Sequence

import numpy as np
import pandas as pd
import tensorflow_datasets as tfds

from astro_datasets.time_series.util import AstroTsDatasetBuilder, AstroTsDatasetInfo

RESOURCES = os.path.join(
    os.path.dirname(__file__), 'resources', 'spcc')

_CITATION = """
@article{kessler2010results,
  title={Results from the supernova photometric classification challenge},
  author={Kessler, Richard and Bassett, Bruce and Belov, Pavel and Bhatnagar, Vasudha and Campbell, Heather and Conley, Alex and Frieman, Joshua A and Glazov, Alexandre and Gonz{\'a}lez-Gait{\'a}n, Santiago and Hlozek, Ren{\'e}e and others},
  journal={Publications of the Astronomical Society of the Pacific},
  volume={122},
  number={898},
  pages={1415},
  year={2010},
  publisher={IOP Publishing}
}
"""

_DESCRIPTION = """
"""


class SPCCDataReader(Sequence):
    """Reader class for SPCC dataset."""

    static_features = [
        'host_photoz', 'mwebv'
    ]

    ts_features = [
        'desg_flux', 'desr_flux', 'desi_flux', 'desz_flux',
    ]

    # Remove any specific instances
    blacklist = [
    ]

    class_keys = {
        1: 0,
        2: 1,
        21: 2,
        22: 3,
        23: 4,
        3: 5,
        32: 6,
        33: 7
    }

    # Time quantisation in days
    time_quantisation = 1.0

    def __init__(self, data_files, metadata_file, remove_no_timeseries=True):
        """Load instances from the SPCC challenge.

        Args:
            data_path: Path containing the records.
            metadata_file: File containing the metadata definitions

        """
        self.static_error_features = [feature + '_error' for feature in self.static_features]
        self.ts_error_features = [feature + '_error' for feature in self.ts_features]
        self.data = pd.concat([pd.read_csv(data_file, header=0, sep=',') for data_file in data_files])
        metadata = pd.read_csv(metadata_file, header=0, sep=',')
        self.metadata = metadata[~metadata['object_id'].isin(self.blacklist)]
        if remove_no_timeseries:
            # Remove an objects we do not have lightcurves for
            self.metadata = metadata[metadata['object_id'].isin(self.data.object_id.unique())]
        for feature in self.static_error_features:
            if feature not in self.metadata:
                self.metadata[feature] = np.nan


    def _quantise_time(self, values):
        return self.time_quantisation * np.round(values / self.time_quantisation)

    def __getitem__(self, index):
        """Get instance at position index of metadata file."""
        instance = self.metadata.iloc[index]
        static = instance[self.static_features]
        static_errors = instance[self.static_error_features]
        object_id = int(instance['object_id'])
        # Read data
        timeseries = self._read_timeseries(object_id)
        time = timeseries['time']
        values = timeseries[self.ts_features]
        value_errors = timeseries[self.ts_error_features]
        instance_class = instance['class']

        return object_id, {
            'static': static,
            'static_errors' : static_errors,
            'time': time,
            'values': values,
            'value_errors': value_errors,
            'targets': {
                'class': self.class_keys[instance_class],
                'sn1a': self.class_keys[instance_class] == 0
            },
            'metadata': {
                'object_id': object_id,
                'redshift': instance['redshift']
            }
        }

    def _read_timeseries(self, object_id):
        data = self.data[self.data['object_id'] == object_id].copy()
        data['time'] = self._quantise_time(data['time'])
        timeseries = data.pivot_table(index='time', columns='parameter', values='value')
        timeseries = timeseries.reindex(columns=self.ts_features + self.ts_error_features).reset_index()
        return timeseries

    def __len__(self):
        """Return number of instances in the dataset."""
        return len(self.metadata)


class SPCC(AstroTsDatasetBuilder):
    """Dataset of the SPCC."""

    VERSION = tfds.core.Version('1.0.10')
    has_metadata = True
    has_timeseries = True
    default_target = 'class'

    def _info(self):
        return AstroTsDatasetInfo(
            builder=self,
            targets={
                'class': tfds.features.ClassLabel(num_classes=8),
                'sn1a': tfds.features.ClassLabel(num_classes=2),
            },
            default_target=self.default_target,
            static_names=SPCCDataReader.static_features,
            timeseries_names=SPCCDataReader.ts_features,
            description=_DESCRIPTION,
            homepage='',
            citation=_CITATION
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Return SplitGenerators."""
        paths = dl_manager.download({
            'train_ts': 'https://storage.googleapis.com/spcc/simgen_training_set.csv.gz',  # noqa: E501
            'train_meta': 'https://storage.googleapis.com/spcc/simgen_training_set_metadata.csv.gz',  # noqa: E501
            'test_ts': 'https://storage.googleapis.com/spcc/simgen_test_set.csv.gz',  # noqa: E501
            'test_meta': 'https://storage.googleapis.com/spcc/simgen_test_set_metadata.csv.gz',  # noqa: E501
        })

        return [
            tfds.core.SplitGenerator(
                name=tfds.Split.TRAIN,
                gen_kwargs={
                    'data_files': [paths['train_ts']],
                    'metadata_file': paths['train_meta']
                },
            ),
            tfds.core.SplitGenerator(
                name=tfds.Split.TEST,
                gen_kwargs={
                    'data_files': [paths['test_ts']],
                    'metadata_file': paths['test_meta']
                }
            )
        ]

    def _generate_examples(self, data_files, metadata_file):
        """Yield examples."""
        reader = SPCCDataReader(data_files, metadata_file)
        for instance in reader:
            yield instance



"""Module containing the Photometric LSST Astronomical Time Series Classification Challenge 2018."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
from collections.abc import Sequence

import numpy as np
import pandas as pd
import tensorflow_datasets as tfds

from astro_datasets.time_series.util import AstroTsDatasetBuilder, AstroTsDatasetInfo

RESOURCES = os.path.join(
    os.path.dirname(__file__), 'resources', 'plasticc')

_CITATION = """
@article{Kessler:2019qge,
    author = "Kessler, R. and others",
    collaboration = "LSST Dark Energy Science, Transient, Variable Stars Science",
    title = "{Models and Simulations for the Photometric LSST Astronomical Time Series Classification Challenge (PLAsTiCC)}",
    eprint = "1903.11756",
    archivePrefix = "arXiv",
    primaryClass = "astro-ph.HE",
    doi = "10.1088/1538-3873/ab26f1",
    journal = "Publ. Astron. Soc. Pac.",
    volume = "131",
    number = "1003",
    pages = "094501",
    year = "2019"
}
"""

_DESCRIPTION = """
"""


class PlasticcDataReader(Sequence):
    """Reader class for Plasticc dataset."""

    static_features = [
        'hostgal_photoz', 'mwebv', 'galactic'
    ]

    ts_features = [
        'lsstu_flux', 'lsstg_flux', 'lsstr_flux', 'lssti_flux', 'lsstz_flux', 'lssty_flux'
    ]

    # Remove any specific instances
    blacklist = [
    ]

    class_keys = {
        6: 0,
        15: 1,
        16: 2,
        42: 3,
        52: 4,
        53: 5,
        62: 6,
        64: 7,
        65: 8,
        67: 9,
        88: 10,
        90: 11,
        92: 12,
        95: 13,
        99: 14,
    }

    # Time quantisation in days
    time_quantisation = 1.0

    def __init__(self, data_files, metadata_file, remove_no_timeseries=True):
        """Load instances from the Plasticc challenge.

        Args:
            data_path: Path containing the records.
            metadata_file: File containing the metadata definitions

        """

        self.static_error_features = [feature + '_error' for feature in self.static_features]
        self.ts_error_features = [feature + '_error' for feature in self.ts_features]
        self.data = pd.concat([pd.read_csv(data_file, header=0, sep=',') for data_file in data_files])
        self.data = self.data.rename(columns={'mjd': 'time', 'flux_err': 'flux_error', })
        self.data = self.data.replace(
            {'passband': {0: 'lsstu', 1: 'lsstg', 2: 'lsstr', 3: 'lssti', 4: 'lsstz', 5: 'lssty', }})
        self.data = pd.melt(self.data, id_vars=['object_id', 'time', 'passband'], value_vars=['flux', 'flux_error'],
                            var_name='parameter', value_name='value')
        self.data['parameter'] = self.data['passband'] + '_' + self.data['parameter']
        self.data.drop('passband', inplace=True, axis=1)
        metadata = pd.read_csv(metadata_file, header=0, sep=',')
        self.metadata = metadata[~metadata['object_id'].isin(self.blacklist)]
        if remove_no_timeseries:
            # Remove an objects we do not have lightcurves for
            self.metadata = metadata[metadata['object_id'].isin(self.data.object_id.unique())]
        self.metadata = self.metadata.rename(columns={'hostgal_photoz_err': 'hostgal_photoz_error'}, )
        self.metadata['galactic'] = (self.metadata['hostgal_photoz'] > 0).astype(float)
        for feature in self.static_error_features:
            if feature not in self.metadata:
                self.metadata[feature] = np.nan


    def _quantise_time(self, values):
        return self.time_quantisation * np.round(values / self.time_quantisation)

    def __getitem__(self, index):
        """Get instance at position index of metadata file."""
        instance = self.metadata.iloc[index]
        static = instance[self.static_features]
        static_errors = instance[self.static_error_features]
        object_id = int(instance['object_id'])
        # Read data
        timeseries = self._read_timeseries(object_id)
        time = timeseries['time']
        values = timeseries[self.ts_features]
        value_errors = timeseries[self.ts_error_features]

        if instance['true_target'] in self.class_keys:
            true_target = self.class_keys[instance['true_target']]
        else:
            true_target = self.class_keys[99]

        return object_id, {
            'static': static,
            'static_errors' : static_errors,
            'time': time,
            'values': values,
            'value_errors': value_errors,
            'targets': {
                'class': true_target,
                'sn1a': true_target == 11
            },
            'metadata': {
                'object_id': object_id,
                'redshift': instance['true_z']
            }
        }

    def _read_timeseries(self, object_id):
        data = self.data[self.data['object_id'] == object_id].copy()
        data['time'] = self._quantise_time(data['time'])
        timeseries = data.pivot_table(index='time', columns='parameter', values='value')
        timeseries = timeseries.reindex(columns=self.ts_features + self.ts_error_features).reset_index()
        return timeseries

    def __len__(self):
        """Return number of instances in the dataset."""
        return len(self.metadata)


class Plasticc(AstroTsDatasetBuilder):
    """Dataset of Plasticc"""

    VERSION = tfds.core.Version('1.0.10')
    has_metadata = True
    has_timeseries = True
    default_target = 'class'

    def _info(self):
        return AstroTsDatasetInfo(
            builder=self,
            targets={
                'class': tfds.features.ClassLabel(num_classes=15),
                'sn1a': tfds.features.ClassLabel(num_classes=2),
            },
            default_target='class',
            static_names=PlasticcDataReader.static_features,
            timeseries_names=PlasticcDataReader.ts_features,
            description=_DESCRIPTION,
            homepage='https://zenodo.org/record/2539456#.XzsWWxNKibs',
            citation=_CITATION
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Return SplitGenerators."""
        paths = dl_manager.download({
            'train_ts': 'https://zenodo.org/record/2539456/files/plasticc_train_lightcurves.csv.gz',  # noqa: E501
            'train_meta': 'https://zenodo.org/record/2539456/files/plasticc_train_metadata.csv.gz',  # noqa: E501
            'test_ts_1': 'https://zenodo.org/record/2539456/files/plasticc_test_lightcurves_01.csv.gz',  # noqa: E501
            'test_meta': 'https://zenodo.org/record/2539456/files/plasticc_test_metadata.csv.gz',  # noqa: E501
        })

        return [
            tfds.core.SplitGenerator(
                name=tfds.Split.TRAIN,
                gen_kwargs={
                    'data_files': [paths['train_ts']],
                    'metadata_file': paths['train_meta']
                },
            ),
            tfds.core.SplitGenerator(
                name=tfds.Split.TEST,
                gen_kwargs={
                    'data_files': [paths['test_ts_1']],
                    'metadata_file': paths['test_meta']
                }
            )
        ]

    def _generate_examples(self, data_files, metadata_file):
        """Yield examples."""
        reader = PlasticcDataReader(data_files, metadata_file)
        for instance in reader:
            yield instance



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import find_packages, setup


def readfile(filename):
    with open(filename, encoding="utf-8") as fp:
        filecontents = fp.read()
    return filecontents

setup(
    name="astro_datasets",
    version="0.0.18",
    description="Astronomy and cosmology tensorflow datasets",
    author="Adam Moss",
    author_email="adam.moss@nottingham.ac.uk",
    maintainer="Adam Moss",
    maintainer_email="adam.moss@nottingham.ac.uk",
    url="https://github.com/adammoss/astro_datasets/",
    license="MIT",
    packages=find_packages(),
    provides=["astro_datasets"],
    install_requires=readfile(
        os.path.join(os.path.dirname(__file__), "requirements.txt")
    )
)
